@@ -30,6 +30,8 @@
 
 #include "pager.h"
 
+#include "egg-screen-help.h"
+
 /* even 16 is pretty darn dubious. */
 #define MAX_REASONABLE_ROWS 16
 #define DEFAULT_ROWS 1
@@ -73,6 +75,8 @@ static void display_help_dialog       (BonoboUIComponent *uic,
 static void display_about_dialog      (BonoboUIComponent *uic,
 				       PagerData         *pager,
 				       const gchar       *verbname);
+static void set_tooltip               (GtkWidget         *applet);
+
 
 static void
 pager_update (PagerData *pager)
@@ -100,6 +104,30 @@ pager_update (PagerData *pager)
 				 pager->display_all);
 }
 
+static WnckScreen *
+applet_get_screen (GtkWidget *applet)
+{
+	int screen_num;
+
+	if (!gtk_widget_has_screen (applet))
+		return wnck_screen_get_default ();
+
+	screen_num = gdk_screen_get_number (gtk_widget_get_screen (applet));
+
+	return wnck_screen_get (screen_num);
+}
+
+static void
+applet_realized (PanelApplet *applet,
+		 PagerData   *pager)
+{
+	WnckScreen *screen;
+
+	screen = applet_get_screen (GTK_WIDGET (applet));
+
+	wnck_pager_set_screen (WNCK_PAGER (pager->pager), screen);
+}
+
 static void
 applet_change_orient (PanelApplet       *applet,
 		      PanelAppletOrient  orient,
@@ -128,6 +156,51 @@ applet_change_orient (PanelApplet       *applet,
 		gtk_label_set_text(GTK_LABEL(pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("Rows") : _("Columns"));	
 }
 
+static void 
+response_cb(GtkWidget * widget,int id, PagerData *pager)
+{
+	if(id == GTK_RESPONSE_HELP) {
+		
+		GError *error = NULL;
+		static GnomeProgram *applet_program = NULL;
+
+		if (!applet_program) {
+			int argc = 1;
+			char *argv[2] = { "workspace-switcher" };
+			applet_program = gnome_program_init ("workspace-switcher", VERSION,
+							     LIBGNOME_MODULE, argc, argv,
+							     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
+		}
+
+		egg_screen_help_display_desktop (
+			gtk_widget_get_screen (pager->applet),
+			applet_program, "workspace-switcher",
+			"workspace-switcher", "workspacelist-prefs", &error);
+		if (error) {
+			GtkWidget *dialog;
+			dialog = gtk_message_dialog_new (GTK_WINDOW(widget),
+							 GTK_DIALOG_DESTROY_WITH_PARENT,
+							 GTK_MESSAGE_ERROR,
+							 GTK_BUTTONS_CLOSE,
+							  _("There was an error displaying help: %s"),
+							 error->message);
+
+			g_signal_connect (G_OBJECT (dialog), "response",
+					  G_CALLBACK (gtk_widget_destroy),
+					  NULL);
+
+			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+			gtk_window_set_screen (GTK_WINDOW (dialog),
+					       gtk_widget_get_screen (pager->applet));
+			gtk_widget_show (dialog);
+			g_error_free (error);
+
+		}
+
+	}
+	else
+		gtk_widget_hide (widget);
+}
 
 static void
 applet_change_pixel_size (PanelApplet *applet,
@@ -343,7 +416,7 @@ fill_pager_applet(PanelApplet *applet)
 		break;
 	}
 
-	pager->screen = wnck_screen_get_default ();
+	pager->screen = applet_get_screen (pager->applet);
 
 	/* because the pager doesn't respond to signals at the moment */
 	wnck_screen_force_update (pager->screen);
@@ -366,8 +439,14 @@ fill_pager_applet(PanelApplet *applet)
 	
 	gtk_container_add (GTK_CONTAINER (pager->applet), pager->frame);
 
+	set_tooltip (GTK_WIDGET (pager->applet));
+
 	gtk_widget_show (pager->applet);
 
+	g_signal_connect (G_OBJECT (pager->applet),
+			  "realize",
+			  G_CALLBACK (applet_realized),
+			  pager);
 	g_signal_connect (G_OBJECT (pager->applet),
 			  "change_orient",
 			  G_CALLBACK (applet_change_orient),
@@ -394,11 +473,39 @@ display_help_dialog (BonoboUIComponent *uic,
 		     const gchar       *verbname)
 {
 	GError *error = NULL;
+	static GnomeProgram *applet_program = NULL;
 
-	gnome_help_display_desktop (NULL, "workspace-switcher", "workspace-switcher", NULL, &error);
+	if (!applet_program) {
+		int argc = 1;
+		char *argv[2] = { "workspace-switcher" };
+		applet_program = gnome_program_init ("workspace-switcher", VERSION,
+						     LIBGNOME_MODULE, argc, argv,
+						     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
+	}
+
+	egg_screen_help_display_desktop (
+		gtk_widget_get_screen (pager->applet),
+		applet_program, "workspace-switcher",
+		"workspace-switcher", NULL, &error);
 	if (error) {
-		g_warning ("help error: %s\n", error->message);
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						  _("There was an error displaying help: %s"),
+						 error->message);
+
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (pager->applet));
+		gtk_widget_show (dialog);
 		g_error_free (error);
+
 	}
 }
 
@@ -421,8 +528,9 @@ display_about_dialog (BonoboUIComponent *uic,
 	};
 	const char *translator_credits = _("translator_credits");
 
-	if (about != NULL) {
-		gtk_widget_show (about);
+	if (about) {
+		gtk_window_set_screen (GTK_WINDOW (about),
+				       gtk_widget_get_screen (pager->applet));
 		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
@@ -440,6 +548,8 @@ display_about_dialog (BonoboUIComponent *uic,
 				 pixbuf);
 	
 	gtk_window_set_wmclass (GTK_WINDOW (about), "pager", "Pager");
+	gtk_window_set_screen (GTK_WINDOW (about),
+			       gtk_widget_get_screen (pager->applet));
 
 	if (pixbuf) {
 		gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
@@ -597,6 +707,27 @@ delete_event (GtkWidget *widget, gpointer data)
 
 #define WID(s) glade_xml_get_widget (xml, s)
 
+static void
+close_dialog (GtkWidget *button,
+              gpointer data)
+{
+	PagerData *pager = data;
+	GtkTreeViewColumn *col;
+
+	/* This is a hack. The "editable" signal for GtkCellRenderer is emitted
+	only on button press or focus cycle. Hence when the user changes the
+	name and closes the preferences dialog without a button-press he would
+	lose the name changes. So, we call the gtk_cell_editable_editing_done
+	to stop the editing. Thanks to Paolo for a better crack than the one I had.
+	*/
+
+	col = gtk_tree_view_get_column(GTK_TREE_VIEW (pager->workspaces_tree),0);
+	if (col->editable_widget != NULL && GTK_IS_CELL_EDITABLE (col->editable_widget))
+	    gtk_cell_editable_editing_done(col->editable_widget);
+
+	gtk_widget_hide (pager->properties_dialog);
+}
+
 static void
 setup_dialog (GladeXML  *xml,
 	      PagerData *pager)
@@ -649,9 +780,12 @@ setup_dialog (GladeXML  *xml,
 	g_signal_connect (pager->properties_dialog, "delete_event",
 			  G_CALLBACK (delete_event),
 			  pager);
+	g_signal_connect (pager->properties_dialog, "response",
+			  G_CALLBACK (response_cb),
+			  pager);
 	
-	g_signal_connect_swapped (WID ("done_button"), "clicked",
-				  (GCallback) gtk_widget_hide, pager->properties_dialog);
+	g_signal_connect (WID ("done_button"), "clicked",
+			  (GCallback) close_dialog, pager);
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_workspaces_spin),
 				   wnck_screen_get_workspace_count (pager->screen));
@@ -701,5 +835,24 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_object_unref (G_OBJECT (xml));
 	}
 
+	gtk_window_set_screen (GTK_WINDOW (pager->properties_dialog),
+			       gtk_widget_get_screen (pager->applet));
 	gtk_window_present (GTK_WINDOW (pager->properties_dialog));
 }
+
+/*
+ *Function to set tooltip
+ */
+static void
+set_tooltip (GtkWidget *applet)
+{
+	GtkTooltips *tooltips;
+
+	tooltips = gtk_tooltips_new ();
+	g_object_ref (tooltips);
+	gtk_object_sink (GTK_OBJECT (tooltips));
+	g_object_set_data_full (G_OBJECT (applet), "tooltips", tooltips,
+                                (GDestroyNotify) g_object_unref);
+        gtk_tooltips_set_tip (tooltips, applet,
+                              _("Workspace Switcher"), NULL);
+}
