@@ -48,32 +48,22 @@ properties_apply_callback(gpointer data)
 						    "pixmap");
 	GtkWidget    *tipentry = g_object_get_data (G_OBJECT (drawer->properties),
 						    "tooltip");
-	char         *s;
 	const char   *cs;
 
 	g_free (drawer->pixmap);
 	drawer->pixmap = NULL;
 	g_free (drawer->tooltip);
 	drawer->tooltip = NULL;
-	s = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (pixentry));
-	if (string_empty (s)) {
+
+	drawer->pixmap = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (pixentry));
+	if (string_empty (drawer->pixmap) ||
+	    access (drawer->pixmap, R_OK) != 0) {
 		drawer->pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
 							    "panel-drawer.png", TRUE, NULL);
-		button_widget_set_pixmap (BUTTON_WIDGET (drawer->button),
-					  drawer->pixmap,-1);
-	} else {
-		if(button_widget_set_pixmap (BUTTON_WIDGET (drawer->button), s, -1))
-			drawer->pixmap = g_strdup (s);
-		else {
-			drawer->pixmap = gnome_program_locate_file (NULL, 
-								    GNOME_FILE_DOMAIN_PIXMAP, 
-								    "panel-drawer.png",
-								     TRUE, NULL);
-			button_widget_set_pixmap (BUTTON_WIDGET (drawer->button),
-						  drawer->pixmap, -1);
-		}
 	}
-	g_free(s);
+
+	button_widget_set_pixmap (BUTTON_WIDGET (drawer->button), drawer->pixmap);
+
 	cs = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(tipentry))));
 	if (string_empty (cs))
 		drawer->tooltip = NULL;
@@ -208,23 +198,14 @@ drawer_click(GtkWidget *w, Drawer *drawer)
 	DrawerWidget *drawerw = DRAWER_WIDGET(drawer->drawer);
 	PanelWidget *parent = PANEL_WIDGET(drawer->button->parent);
 	GtkWidget *panelw = parent->panel_parent;
-#ifdef DRAWER_DEBUG
-	printf ("Registering drawer click \n");
-#endif	
 
 	switch (BASEP_WIDGET (drawerw)->state) {
 	case BASEP_SHOWN:
 	case BASEP_AUTO_HIDDEN:
-#ifdef DRAWER_DEBUG
-	printf ("Drawer closing\n");
-#endif
 		drawer_widget_close_drawer (drawerw, panelw);
 		break;
 	case BASEP_HIDDEN_LEFT:
 	case BASEP_HIDDEN_RIGHT:
-#ifdef DRAWER_DEBUG
-	printf ("Drawer opening\n");
-#endif
 		drawer_widget_open_drawer (drawerw, panelw);
 		break;
 	}
@@ -289,8 +270,6 @@ focus_in_drawer (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
 	Drawer *drawer = data;
 	DrawerWidget *drawerw = DRAWER_WIDGET(drawer->drawer);
-	PanelWidget *parent = PANEL_WIDGET(drawer->button->parent);
-	GtkWidget *panelw = parent->panel_parent;
 
 	switch (BASEP_WIDGET (drawerw)->state) {
 	case BASEP_HIDDEN_LEFT:
@@ -311,8 +290,6 @@ focus_out_drawer (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
 	Drawer *drawer = data;
 	DrawerWidget *drawerw = DRAWER_WIDGET(drawer->drawer);
-	PanelWidget *parent = PANEL_WIDGET(drawer->button->parent);
-	GtkWidget *panelw = parent->panel_parent;
 
 	switch (BASEP_WIDGET (drawerw)->state) {
 	case BASEP_SHOWN:
@@ -345,32 +322,32 @@ key_press_drawer (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		switch (event->keyval) {
 		case GDK_Up:
 		case GDK_KP_Up:
-			if (parent->orient == GTK_ORIENTATION_HORIZONTAL)
-				dir = GTK_DIR_TAB_BACKWARD;
-			else
-				return TRUE;
-			break;
 		case GDK_Left:
 		case GDK_KP_Left:
-			if (parent->orient == GTK_ORIENTATION_VERTICAL)
+			switch (BASEP_WIDGET (drawerw)->state) {
+			case BASEP_SHOWN:
+			case BASEP_AUTO_HIDDEN:
 				dir = GTK_DIR_TAB_BACKWARD;
-			else
-				return TRUE;
+				break;
+			default:
+                                return TRUE;
+				break;
+			}
 			break;
 		case GDK_Down:
 		case GDK_KP_Down:
-			if (parent->orient == GTK_ORIENTATION_HORIZONTAL)
-				dir = GTK_DIR_TAB_FORWARD;
-			else
-				return TRUE;
-			break;
 		case GDK_Right:
 		case GDK_KP_Right:
-			if (parent->orient == GTK_ORIENTATION_VERTICAL)
+			switch (BASEP_WIDGET (drawerw)->state) {
+			case BASEP_SHOWN:
+			case BASEP_AUTO_HIDDEN:
 				dir = GTK_DIR_TAB_FORWARD;
-			else
-				return TRUE;
-			break;
+				break;
+			default:
+                                return TRUE;
+				break;
+			}
+                        break;
 		case GDK_Escape:
 			switch (BASEP_WIDGET (drawerw)->state) {
 			case BASEP_SHOWN:
@@ -397,6 +374,48 @@ key_press_drawer (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	}
 }
 
+/*
+ * This function implements Esc moving focus from the drawer to the drawer
+ * icon and closing the drawer and Shift+Esc moving focus from the drawer
+ * to the drawer icon without closing the drawer when focus is in the drawer.
+ */
+static gboolean
+key_press_drawer_widget (GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	Drawer *drawer;
+	DrawerWidget *drawerw;
+	PanelWidget *parent;
+	gboolean ret_val = FALSE;
+
+	switch (event->keyval) {
+	case GDK_Escape: {
+		drawer = data;
+		parent = PANEL_WIDGET (drawer->button->parent);
+		drawerw = DRAWER_WIDGET (drawer->drawer);
+
+		if (event->state == GDK_SHIFT_MASK) {
+			gtk_window_present (GTK_WINDOW (parent->panel_parent));
+			ret_val = TRUE;
+		} else if (event->state == 0) {
+			gtk_window_present (GTK_WINDOW (parent->panel_parent));
+			switch (BASEP_WIDGET (drawerw)->state) {
+			case BASEP_SHOWN:
+			case BASEP_AUTO_HIDDEN:
+				drawer_widget_close_drawer (drawerw, parent->panel_parent);
+				break;
+			default:
+				break;
+			}
+			ret_val = TRUE;
+			break;
+		} 
+	}
+	default:
+		break;
+	}
+	return ret_val;
+}
+
 static void  
 drag_data_get_cb (GtkWidget          *widget,
 		  GdkDragContext     *context,
@@ -416,10 +435,23 @@ drag_data_get_cb (GtkWidget          *widget,
 	g_free (foo);
 }
 
+static void
+free_drawer (Drawer *drawer)
+{
+	if (drawer->tooltip)
+		g_free (drawer->tooltip);
+
+	if (drawer->pixmap)
+		g_free (drawer->pixmap);
+
+	g_free (drawer);
+}
+
 static Drawer *
-create_drawer_applet(GtkWidget * drawer_panel,
-		     const char *tooltip, const char *pixmap,
-		     PanelOrient orient)
+create_drawer_applet (GtkWidget   *drawer_panel,
+		      const char  *tooltip,
+		      const char  *pixmap,
+		      PanelOrient  orient)
 {
         static GtkTargetEntry dnd_targets[] = {
 		{ "application/x-panel-applet-internal", 0, 0 }
@@ -442,8 +474,11 @@ create_drawer_applet(GtkWidget * drawer_panel,
 		drawer->pixmap = g_strdup (pixmap);
 	}
 	drawer->button = button_widget_new (drawer->pixmap, -1,
-					    TRUE, orient,
-					    _("Drawer"));
+					    TRUE, orient);
+	if (!drawer->button) {
+		free_drawer (drawer);
+		return NULL;
+	}
 
 	/*A hack since this function only pretends to work on window
 	  widgets (which we actually kind of are) this will select
@@ -481,6 +516,8 @@ create_drawer_applet(GtkWidget * drawer_panel,
 			    G_CALLBACK (focus_out_drawer), drawer);
 	g_signal_connect (G_OBJECT (drawer->button), "key_press_event",
 			    G_CALLBACK (key_press_drawer), drawer);
+	g_signal_connect (G_OBJECT (drawer->drawer), "key_press_event",
+			    G_CALLBACK (key_press_drawer_widget), drawer);
 
 	g_object_set_data (G_OBJECT (drawer_panel), DRAWER_PANEL_KEY, drawer);
 	gtk_widget_queue_resize (GTK_WIDGET (drawer_panel));
@@ -489,17 +526,28 @@ create_drawer_applet(GtkWidget * drawer_panel,
 }
 
 static Drawer *
-create_empty_drawer_applet(const char *tooltip, const char *pixmap,
-			   PanelOrient orient)
+create_empty_drawer_applet (PanelWidget *container,
+			    const char  *tooltip,
+			    const char  *pixmap,
+			    PanelOrient  orient)
 {
-	GtkWidget *dw = drawer_widget_new (NULL, orient,
+	GtkWidget *drawer_widget;
+	int        screen;
+	int        monitor;
+
+	screen  = panel_screen_from_panel_widget (container);
+	monitor = panel_monitor_from_panel_widget (container);
+
+	drawer_widget = drawer_widget_new (NULL, screen,
+					   monitor, orient,
 					   BASEP_EXPLICIT_HIDE,
 					   BASEP_SHOWN,
 					   PANEL_SIZE_MEDIUM,
 					   TRUE, TRUE,
 					   PANEL_BACK_NONE, NULL,
 					   TRUE, FALSE, TRUE, NULL);
-	return create_drawer_applet (dw, tooltip, pixmap, orient);
+	return create_drawer_applet (
+			drawer_widget, tooltip, pixmap, orient);
 }
 
 void
@@ -557,7 +605,7 @@ load_drawer_applet (gchar       *mypanel_id,
 	orient = panel_widget_get_applet_orient (panel);
 
 	if (!mypanel_id) {
-		drawer = create_empty_drawer_applet (tooltip, pixmap, orient);
+		drawer = create_empty_drawer_applet (panel, tooltip, pixmap, orient);
 		if (drawer != NULL)
 			panel_setup (drawer->drawer);
 		panels_to_sync = TRUE;
@@ -568,12 +616,12 @@ load_drawer_applet (gchar       *mypanel_id,
 
 		if (dr_pd == NULL) {
 			g_warning ("Can't find the panel for drawer, making a new panel");
-			drawer = create_empty_drawer_applet(tooltip, pixmap, orient);
+			drawer = create_empty_drawer_applet(panel, tooltip, pixmap, orient);
 			if(drawer) panel_setup(drawer->drawer);
 			panels_to_sync = TRUE;
 		} else if ( ! DRAWER_IS_WIDGET (dr_pd->panel)) {
 			g_warning ("I found a bogus panel for a drawer, making a new one");
-			drawer = create_empty_drawer_applet(tooltip, pixmap, orient);
+			drawer = create_empty_drawer_applet(panel, tooltip, pixmap, orient);
 			if(drawer) panel_setup(drawer->drawer);
 			panels_to_sync = TRUE;
 		} else {
@@ -593,7 +641,7 @@ load_drawer_applet (gchar       *mypanel_id,
 
 		drawer->info = panel_applet_register (
 					drawer->button, drawer,
-					(GDestroyNotify) g_free,
+					(GDestroyNotify) free_drawer,
 					panel, pos, exactpos,
 					APPLET_DRAWER, gconf_key);
 
@@ -621,15 +669,16 @@ load_drawer_applet (gchar       *mypanel_id,
 	gtk_tooltips_set_tip (panel_tooltips,drawer->button,
 			      drawer->tooltip,NULL);
 	drawer_setup (drawer);
+	gtk_window_present (GTK_WINDOW (drawer->drawer));
 
 	if (!commie_mode)
 		panel_applet_add_callback (drawer->info,
 					   "properties",
 					   GTK_STOCK_PROPERTIES,
-					   _("Properties..."));
+					   _("_Properties"));
 
 	panel_applet_add_callback (
-		drawer->info, "help", GTK_STOCK_HELP, _("Help"));
+		drawer->info, "help", GTK_STOCK_HELP, _("_Help"));
 
 	return drawer;
 }
@@ -669,6 +718,8 @@ drawer_save_to_gconf (Drawer     *drawer,
 				PANEL_GCONF_OBJECTS, profile, gconf_key, "tooltip");
 		gconf_client_set_string (client, temp_key, drawer->tooltip, NULL);
 	}
+
+	panel_save_to_gconf (g_object_get_data (G_OBJECT (drawer->drawer), "PanelData"));
 }
 
 void
