@@ -5,12 +5,12 @@
 #include <gtk/gtktypeutils.h>
 #include "GNOME_Panel.h"
 
-typedef GNOME_Vertigo_PanelOrient PanelOrient;
-
-#define PANEL_ORIENT_UP    GNOME_Vertigo_PANEL_ORIENT_UP
-#define PANEL_ORIENT_DOWN  GNOME_Vertigo_PANEL_ORIENT_DOWN
-#define PANEL_ORIENT_LEFT  GNOME_Vertigo_PANEL_ORIENT_LEFT
-#define PANEL_ORIENT_RIGHT GNOME_Vertigo_PANEL_ORIENT_RIGHT
+typedef enum {
+	PANEL_ORIENT_UP    = GNOME_Vertigo_PANEL_ORIENT_UP,
+	PANEL_ORIENT_DOWN  = GNOME_Vertigo_PANEL_ORIENT_DOWN,
+	PANEL_ORIENT_LEFT  = GNOME_Vertigo_PANEL_ORIENT_LEFT,
+	PANEL_ORIENT_RIGHT = GNOME_Vertigo_PANEL_ORIENT_RIGHT,
+} PanelOrient;
 
 typedef enum {
 	PANEL_SIZE_XX_SMALL = GNOME_Vertigo_PANEL_XX_SMALL,
@@ -28,12 +28,6 @@ typedef enum {
 	PANEL_BACK_PIXMAP
 } PanelBackType;
 
-typedef enum {
-	LAYER_BELOW,
-	LAYER_NORMAL,
-	LAYER_ABOVE
-} PanelLayer;
-
 /* XXX: if you add any here you need to make the tile type larger
  * for button-widget, as it's 2 bits only for now */
 
