@@ -15,6 +15,8 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 
+#include <gdk/gdkkeysyms.h>
+
 #include <libgnome/libgnome.h>
 #include <libbonobo.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -35,26 +37,24 @@
 #include "foobar-widget.h"
 #include "gnome-run.h"
 #include "launcher.h"
-#include "logout.h"
 #include "menu-fentry.h"
 #include "menu-util.h"
 #include "menu.h"
+#include "multiscreen-stuff.h"
 #include "panel-util.h"
 #include "panel-config.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
 #include "session.h"
-#include "status.h"
 #include "panel-applet-frame.h"
 #include "global-keys.h"
+#include "panel-action-button.h"
 
 #define PANEL_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
 			   GDK_BUTTON_RELEASE_MASK |		\
 			   GDK_POINTER_MOTION_MASK |		\
 			   GDK_POINTER_MOTION_HINT_MASK)
 
-#undef PANEL_SESSION_DEBUG
-
 /*list of all panel widgets created*/
 GSList *panel_list = NULL;
 
@@ -159,25 +159,9 @@ static GConfEnumStringPair panel_speed_type_enum_map [] = {
 	{ PANEL_SPEED_FAST,   "panel-speed-fast" },
 };
 
-static GConfEnumStringPair panel_layer_type_enum_map [] = {
-	{ LAYER_NORMAL, "panel-normal-layer" },
-	{ LAYER_BELOW, "panel-below-layer" },
-	{ LAYER_ABOVE, "panel-above-layer" },
-};
-
-static void
-change_window_cursor(GdkWindow *window, GdkCursorType cursor_type)
-{
-	GdkCursor *cursor = gdk_cursor_new (cursor_type);
-	gdk_window_set_cursor (window, cursor);
-	gdk_cursor_unref (cursor);
-}
-
 static void
 panel_realize (GtkWidget *widget, gpointer data)
 {
-	change_window_cursor (widget->window, GDK_LEFT_PTR);
-	
 	if (BASEP_IS_WIDGET (widget))
 		basep_widget_enable_buttons(BASEP_WIDGET(widget), TRUE);
 	else if (FOOBAR_IS_WIDGET (widget))
@@ -217,15 +201,6 @@ orientation_change (AppletInfo  *info,
 				       (gpointer)basep->panel);
 		}
 		break;
-	case APPLET_STATUS: {
-		StatusApplet *status = info->data;
-
-		if (status->orient != panel->orient) {
-			status->orient = panel->orient;
-			status_applet_update (status);
-		}
-		}
-		break;
 	default:
 		break;
 	}
@@ -242,9 +217,7 @@ orient_change_foreach(GtkWidget *w, gpointer data)
 
 
 static void
-panel_orient_change(GtkWidget *widget,
-		    GtkOrientation orient,
-		    gpointer data)
+panel_orient_change (GtkWidget *widget, gpointer data)
 {
 	gtk_container_foreach(GTK_CONTAINER(widget),
 			      orient_change_foreach,
@@ -277,24 +250,9 @@ void
 size_change (AppletInfo  *info,
 	     PanelWidget *panel)
 {
-	PanelSize size = panel->sz;
-	
-	switch (info->type) {
-	case APPLET_BONOBO:
+	if (info->type == APPLET_BONOBO)
 		panel_applet_frame_change_size (
-			PANEL_APPLET_FRAME (info->widget), size);
-		break;
-	case APPLET_STATUS: {
-		StatusApplet *status = info->data;
-
-		status->size = size;
-
-		status_applet_update (status);
-		}
-		break;
-	default:
-		break;
-	}
+			PANEL_APPLET_FRAME (info->widget), panel->sz);
 }
 
 static void
@@ -308,9 +266,7 @@ size_change_foreach(GtkWidget *w, gpointer data)
 
 
 static void
-panel_size_change(GtkWidget *widget,
-		  int sz,
-		  gpointer data)
+panel_size_change (GtkWidget *widget, gpointer data)
 {
 	gtk_container_foreach(GTK_CONTAINER(widget), size_change_foreach,
 			      widget);
@@ -340,10 +296,7 @@ back_change_foreach (GtkWidget   *widget,
 }
 
 static void
-panel_back_change(GtkWidget *widget,
-		  PanelBackType type,
-		  char *pixmap,
-		  GdkColor *color)
+panel_back_change (GtkWidget *widget, gpointer data)
 {
 	gtk_container_foreach (GTK_CONTAINER (widget),
 			       (GtkCallback) back_change_foreach,
@@ -396,8 +349,8 @@ state_hide_foreach(GtkWidget *w, gpointer data)
 		   and all that will get updated in the main loop */
 		if(widget->window) {
 			gdk_window_move(widget->window,
-					-widget->allocation.width - 1,
-					-widget->allocation.height - 1);
+					-ABS (widget->allocation.width) - 1,
+					-ABS (widget->allocation.height) - 1);
 		}
 	}
 }
@@ -533,19 +486,28 @@ menu_deactivate(GtkWidget *w, PanelData *pd)
 }
 
 static void
-move_panel_to_cursor(GtkWidget *w)
+move_panel_to_cursor (GtkWidget *panel)
 {
-	int x,y;
-	gdk_window_get_pointer(NULL,&x,&y,NULL);
-	if(BASEP_IS_WIDGET(w))
-		basep_widget_set_pos(BASEP_WIDGET(w),x,y);
+	GdkScreen *screen;
+	GdkWindow *root_window;
+	int        x, y;
+
+	g_return_if_fail (GTK_IS_WINDOW (panel));
+
+	screen = gtk_window_get_screen (GTK_WINDOW (panel));
+	root_window = gdk_screen_get_root_window (screen);
+
+	gdk_window_get_pointer (root_window, &x, &y, NULL);
+
+	if (BASEP_IS_WIDGET (panel))
+		basep_widget_set_pos (BASEP_WIDGET (panel), x, y);
 }
 
 static gboolean
-panel_move_timeout(gpointer data)
+panel_move_timeout (GtkWidget *panel)
 {
-	if(panel_dragged && panel_been_moved)
-		move_panel_to_cursor(data);
+	if (panel_dragged && panel_been_moved)
+		move_panel_to_cursor (panel);
 	
 	panel_been_moved = FALSE;
 	panel_dragged_timeout = 0;
@@ -565,9 +527,9 @@ panel_remove_applets (PanelWidget *panel)
 		info = g_object_get_data (
 				G_OBJECT (applet_data->applet), "applet_info");
 
-		if (info && info->type == APPLET_BONOBO)
-			panel_applet_frame_set_clean_remove (
-					PANEL_APPLET_FRAME (info->data), TRUE);
+		if (info && info->type == APPLET_LAUNCHER)
+			launcher_properties_destroy (info->data);
+			
 	}
 }
 
@@ -584,30 +546,14 @@ panel_destroy (GtkWidget *widget, gpointer data)
 
 	panel_remove_applets (panel);
 		
-	kill_config_dialog(widget);
+	kill_config_dialog (widget);
 
-	if (DRAWER_IS_WIDGET(widget)) {
-		GtkWidget *master_widget = panel->master_widget;
-
-		if (master_widget != NULL) {
-			AppletInfo *info =
-				g_object_get_data (G_OBJECT (master_widget),
-						   "applet_info");
-			Drawer *drawer = info->data;
-			drawer->drawer = NULL;
-			panel_applet_clean (info);
-			g_object_set_data (G_OBJECT (master_widget),
-					   "applet_info", NULL);
-		}
-	} else if ((BASEP_IS_WIDGET(widget)
-		    && !DRAWER_IS_WIDGET(widget))
-		   || FOOBAR_IS_WIDGET (widget)) {
-		/*this is a base panel and we just lost it*/
+	if ((BASEP_IS_WIDGET (widget) && !DRAWER_IS_WIDGET (widget)) ||
+	    FOOBAR_IS_WIDGET (widget))
 		base_panels--;
-	}
 
-	if (pd->menu != NULL)
-		g_object_unref (G_OBJECT (pd->menu));
+	if (pd->menu)
+		g_object_unref (pd->menu);
 	pd->menu = NULL;
 
 	pd->panel = NULL;
@@ -634,22 +580,6 @@ panel_applet_move(PanelWidget *panel, GtkWidget *widget, gpointer data)
 	panel_applet_save_position (info, info->gconf_key, FALSE);
 }
 
-static void
-panel_applet_about_to_die (GtkWidget *panel,
-			   GtkWidget *widget,
-			   gpointer   data)
-{
-	AppletInfo *info;
-
-	info = g_object_get_data (G_OBJECT (widget), "applet_info");
-
-	g_return_if_fail (info);
-
-	/*
-	 * FIXME: we need to implement an applet died dialog box
-	 */
-}
-
 static GtkWidget *
 panel_menu_get (PanelWidget *panel, PanelData *pd)
 {
@@ -688,55 +618,55 @@ make_popup_panel_menu (PanelWidget *panel)
 }
 
 static gboolean
-panel_initiate_move (GtkWidget *widget, guint32 event_time)
+panel_initiate_move (GtkWidget *widget,
+		     guint32    event_time)
 {
-	PanelWidget *panel = NULL;
-	BasePWidget *basep = NULL;
+	PanelWidget *panel;
+	BasePWidget *basep;
+	GdkCursor   *cursor;
 
-	if (BASEP_IS_WIDGET (widget)) {
-		basep = BASEP_WIDGET (widget);
-		panel = PANEL_WIDGET (basep->panel);
-	} else if (FOOBAR_IS_WIDGET (widget)) {
-		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
-	}
+	if (FOOBAR_IS_WIDGET (widget))
+		return FALSE;
 
-	/*this should probably be in snapped widget*/
-	if(!panel_dragged &&
-	   !DRAWER_IS_WIDGET (widget) &&
-	   !FOOBAR_IS_WIDGET (widget)) {
-		GdkCursor *cursor = gdk_cursor_new (GDK_FLEUR);
-		gtk_grab_add(widget);
-		gdk_pointer_grab (widget->window,
-				  FALSE,
-				  PANEL_EVENT_MASK,
-				  NULL,
-				  cursor,
-				  event_time);
-		gdk_cursor_unref (cursor);
+	g_assert (BASEP_IS_WIDGET (widget));
 
-		if (basep) {
-			basep->autohide_inhibit = TRUE;
-			basep_widget_init_offsets (basep);
-		}
+	basep = BASEP_WIDGET (widget);
+	panel = PANEL_WIDGET (basep->panel);
 
-		panel_dragged = TRUE;
-		return TRUE;
-	} if(DRAWER_IS_WIDGET(widget) &&
-	     !panel_applet_in_drag) {
+	if (DRAWER_IS_WIDGET (widget) && !panel_applet_in_drag) {
 		panel_widget_applet_drag_start (
-						PANEL_WIDGET(panel->master_widget->parent),
-						panel->master_widget,
-						PW_DRAG_OFF_CURSOR);
+			PANEL_WIDGET (panel->master_widget->parent),
+			panel->master_widget,
+			PW_DRAG_OFF_CURSOR);
 		return TRUE;
 	}
 
-	return FALSE;
+	if (panel_dragged)
+		return FALSE;
+
+	gtk_grab_add (widget);
+
+	cursor = gdk_cursor_new (GDK_FLEUR);
+	gdk_pointer_grab (widget->window, FALSE, PANEL_EVENT_MASK,
+			  NULL, cursor, event_time);
+	gdk_cursor_unref (cursor);
+
+	basep->autohide_inhibit = TRUE;
+	basep_widget_init_offsets (basep);
+	panel_dragged = TRUE;
+
+	return TRUE;
 }
 
 static gboolean
-panel_do_popup_menu (PanelWidget *panel, BasePWidget *basep, GtkWidget *widget, guint button, guint32 activate_time)
+panel_do_popup_menu (PanelWidget *panel,
+		     BasePWidget *basep,
+		     GtkWidget   *widget,
+		     int          screen,
+		     guint        button,
+		     guint32      activate_time)
 {
-	if(!panel_applet_in_drag) {
+	if (!panel_applet_in_drag) {
 		GtkWidget *menu;
 
 		menu = make_popup_panel_menu (panel);
@@ -745,6 +675,9 @@ panel_do_popup_menu (PanelWidget *panel, BasePWidget *basep, GtkWidget *widget,
 			basep_widget_autohide (basep);
 		}
 
+		gtk_menu_set_screen (GTK_MENU (menu),
+				     panel_screen_from_number (screen));
+
 		gtk_menu_popup (GTK_MENU (menu),
                                 NULL,
                                 NULL,
@@ -754,6 +687,7 @@ panel_do_popup_menu (PanelWidget *panel, BasePWidget *basep, GtkWidget *widget,
                                 activate_time);
 		return TRUE;
 	}
+
 	return FALSE;
 }
 
@@ -762,72 +696,43 @@ panel_popup_menu (GtkWidget *widget, gpointer data)
 {
  	PanelWidget *panel = NULL;
 	BasePWidget *basep = NULL;
+	int          screen = 0;
 
 	panel = PANEL_WIDGET (widget);
 	if (BASEP_IS_WIDGET (data)) {
 		basep = BASEP_WIDGET (data);
-	}
-	return panel_do_popup_menu (panel, basep, widget, 3, GDK_CURRENT_TIME);
-}
+		screen = basep->screen;
+	} else if (FOOBAR_IS_WIDGET (data))
+		screen = FOOBAR_WIDGET (data)->screen;
 
-static gboolean
-pointer_in_widget (GtkWidget *widget, GdkEventButton *event)
-{
-	int x, y;
-	int wx, wy;
-	int width, height;
-
-	x = (int) event->x_root;
-	y = (int) event->y_root;
-	gdk_window_get_origin (widget->window, &wx, &wy);
-	width = widget->allocation.width;
-	height = widget->allocation.height;
-	if ((x < wx || x >= wx + width) ||
-            (y < wy || y >= wy + height))
-		return FALSE;
-	else
-		return TRUE;
-}
-
-static void
-queue_resize_button_widgets (GtkWidget *w, gpointer data)
-{
-	if (BUTTON_IS_WIDGET (w)) {
-		gtk_widget_queue_resize (w);
-	}
+	return panel_do_popup_menu (
+			panel, basep, widget, screen, 3, GDK_CURRENT_TIME);
 }
 
 static gboolean
 panel_end_move (GtkWidget *widget, GdkEventButton *bevent)
 {
-	if (panel_dragged &&
-	    BASEP_IS_WIDGET (widget)) {
-		BasePWidget *basep = BASEP_WIDGET (widget);
-		basep_widget_set_pos (basep,
-				      (gint16)bevent->x_root, 
-				      (gint16)bevent->y_root);
-		basep->autohide_inhibit = FALSE;
-		basep_widget_queue_autohide (basep);
+	BasePWidget *basep;
 
-		gtk_grab_remove (widget);
-		gdk_pointer_ungrab (bevent->time);
-		panel_dragged = FALSE;
-		if (panel_dragged_timeout != 0)
-			g_source_remove (panel_dragged_timeout);
-		panel_dragged_timeout = 0;
-		panel_been_moved = FALSE;
-		if (pointer_in_widget (basep->panel, bevent) &&
-		    ! gtk_widget_is_focus (basep->panel))
-			panel_widget_focus (PANEL_WIDGET (basep->panel));
+	if (!panel_dragged || !BASEP_IS_WIDGET (widget))
+		return FALSE;
 
-		/* FIXME: why is this neccessary!!!!???? */
-		gtk_container_foreach (GTK_CONTAINER (basep->panel),
-				       queue_resize_button_widgets,
-				       NULL);
+	basep = BASEP_WIDGET (widget);
 
-		return TRUE;
-	}
-	return FALSE;
+	basep_widget_set_pos (basep, bevent->x_root, bevent->y_root);
+	basep->autohide_inhibit = FALSE;
+	basep_widget_queue_autohide (basep);
+
+	gtk_grab_remove (widget);
+	gdk_pointer_ungrab (bevent->time);
+	panel_dragged = FALSE;
+
+	if (panel_dragged_timeout != 0)
+		g_source_remove (panel_dragged_timeout);
+	panel_dragged_timeout = 0;
+	panel_been_moved = FALSE;
+
+	return TRUE;
 }
 
 static gboolean
@@ -837,13 +742,17 @@ panel_event(GtkWidget *widget, GdkEvent *event)
 	PanelWidget *panel = NULL;
 	BasePWidget *basep = NULL;
 	GdkEventButton *bevent;
+	GdkEventKey *kevent;
 	int x, y;
+	int screen = 0;
 
 	if (BASEP_IS_WIDGET (widget)) {
 		basep = BASEP_WIDGET (widget);
 		panel = PANEL_WIDGET (basep->panel);
+		screen = basep->screen;
 	} else if (FOOBAR_IS_WIDGET (widget)) {
 		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
+		screen= FOOBAR_WIDGET (widget)->screen;
 	}
 
 	switch (event->type) {
@@ -863,11 +772,12 @@ panel_event(GtkWidget *widget, GdkEvent *event)
 			else
 				pd->insertion_pos = x;
 
-			if (panel_do_popup_menu (panel, basep, widget, bevent->button, bevent->time))
+			if (panel_do_popup_menu (panel, basep, widget, screen,
+						 bevent->button, bevent->time))
 				return TRUE;
 			break;
 		case 2:
-			if ( ! commie_mode)
+			if (!commie_mode)
 				return panel_initiate_move (widget,
 							    bevent->time);
 			break;
@@ -877,22 +787,32 @@ panel_event(GtkWidget *widget, GdkEvent *event)
 
 	case GDK_BUTTON_RELEASE:
 		bevent = (GdkEventButton *) event;
-		if (panel_dragged) {
+		if (panel_dragged)
 			return panel_end_move (widget, bevent);
-		}
 		break;
 	case GDK_MOTION_NOTIFY:
 		if (panel_dragged) {
-			if (panel_dragged_timeout == 0) {
+			if (!panel_dragged_timeout) {
 				panel_been_moved = FALSE;
 				move_panel_to_cursor (widget);
-				panel_dragged_timeout = g_timeout_add (30, panel_move_timeout, widget);
-			} else {
+				panel_dragged_timeout = g_timeout_add (30, (GSourceFunc) panel_move_timeout, widget);
+			} else
 				panel_been_moved = TRUE;
-			}
 		}
 		break;
-
+	case GDK_KEY_PRESS:
+		kevent = (GdkEventKey *)event;
+		if (GTK_IS_SOCKET (GTK_WINDOW (widget)->focus_widget)) {
+			/*
+		  	 * If the focus widget is a GtkSocket, i.e. the
+			 * focus is in an applet in another process, then key 
+			 * bindings do not work. We get around this by
+			 * activating the key binding we require here.
+			 */ 
+			if (kevent->keyval == GDK_F10 && kevent->state == GDK_CONTROL_MASK)
+				return gtk_bindings_activate (GTK_OBJECT (panel), kevent->keyval, kevent->state);
+		}
+		break;
 	default:
 		break;
 	}
@@ -976,16 +896,6 @@ drop_menu (PanelWidget *panel, int pos, const char *dir)
 	load_menu_applet (dir, FALSE /* main_menu */, flags, TRUE, FALSE, NULL, panel, pos, TRUE, NULL);
 }
 
-static gboolean
-uri_exists (const char *uri)
-{
-	gboolean ret;
-	GnomeVFSURI *vfs_uri = gnome_vfs_uri_new (uri);
-	ret = gnome_vfs_uri_exists (vfs_uri);
-	gnome_vfs_uri_unref (vfs_uri);
-	return ret;
-}
-
 static void
 drop_nautilus_uri (PanelWidget *panel,
 		   int pos,
@@ -1019,7 +929,7 @@ drop_directory (PanelWidget *panel, int pos, const char *dir)
 	/* not filename, but path, these are uris, not local
 	 * files */
 	tmp = g_build_path ("/", dir, ".directory", NULL);
-	if (uri_exists (tmp)) {
+	if (panel_uri_exists (tmp)) {
 		g_free (tmp);
 		drop_menu (panel, pos, dir);
 		return;
@@ -1027,7 +937,7 @@ drop_directory (PanelWidget *panel, int pos, const char *dir)
 	g_free (tmp);
 
 	tmp = g_build_path ("/", dir, ".order", NULL);
-	if (uri_exists (tmp)) {
+	if (panel_uri_exists (tmp)) {
 		g_free (tmp);
 		drop_menu (panel, pos, dir);
 		return;
@@ -1124,6 +1034,7 @@ drop_urilist (PanelWidget *panel, int pos, char *urilist,
 			g_free (menu_uri);
 		} else if (mimetype != NULL &&
 			   (strcmp(mimetype, "application/x-gnome-app-info") == 0 ||
+			    strcmp(mimetype, "application/x-desktop") == 0 ||
 			    strcmp(mimetype, "application/x-kde-app-info") == 0)) {
 			Launcher *launcher;
 			
@@ -1266,36 +1177,23 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 		else
 			drop_menu (panel, pos, menu);
 
-	} else if (strcmp(applet_type,"DRAWER:NEW")==0) {
-		load_drawer_applet(NULL, NULL, NULL, panel, pos, TRUE, NULL);
+	} else if (!strcmp (applet_type, "DRAWER:NEW"))
+		load_drawer_applet (NULL, NULL, NULL, panel, pos, TRUE, NULL);
 
-	} else if (strcmp (applet_type, "LOGOUT:NEW") == 0) {
-		load_logout_applet (panel, pos, TRUE, NULL);
+	else if (!strncmp (applet_type, "ACTION:", strlen ("ACTION:")))
+		remove_applet = panel_action_button_load_from_drag (
+					applet_type, panel, pos,
+					TRUE, NULL, &applet_num);
 
-	} else if (sscanf (applet_type, "LOGOUT:%d", &applet_num) == 1) {
-		load_logout_applet (panel, pos, TRUE, NULL);
-		remove_applet = TRUE;
-
-	} else if (strcmp (applet_type, "LOCK:NEW") == 0) {
-		load_lock_applet (panel, pos, TRUE, NULL);
-
-	} else if (sscanf (applet_type, "LOCK:%d", &applet_num) == 1) {
-		load_lock_applet (panel, pos, TRUE, NULL);
-		remove_applet = TRUE;
-
-	} else if(strcmp(applet_type,"LAUNCHER:ASK")==0) {
-		ask_about_launcher(NULL, panel, pos, TRUE);
-
-	} else if(strcmp(applet_type,"STATUS:TRY")==0) {
-		load_status_applet(panel, pos, TRUE, NULL);
-	} 
+	else if (!strcmp(applet_type,"LAUNCHER:ASK"))
+		ask_about_launcher (NULL, panel, pos, TRUE);
 
 	if (remove_applet &&
 	    action == GDK_ACTION_MOVE) {
 		AppletInfo *info = g_slist_nth_data (applets, applet_num);
 
 		if (info != NULL)
-			panel_applet_clean (info);
+			panel_applet_clean (info, TRUE);
 	}
 }
 
@@ -1568,7 +1466,7 @@ drag_data_recieved_cb (GtkWidget	*widget,
 			return;
 		}
 		panel_applet_frame_load ((char *)selection_data->data,
-					 panel, pos, NULL);
+					 panel, pos, TRUE, NULL);
 		break;
 	case TARGET_APPLET_INTERNAL:
 		drop_internal_applet (panel, pos, (char *)selection_data->data,
@@ -1601,17 +1499,13 @@ panel_widget_setup(PanelWidget *panel)
 			  "applet_move",
 			  G_CALLBACK(panel_applet_move),
 			  NULL);
-	g_signal_connect (G_OBJECT(panel),
-			  "applet_about_to_die",
-			  G_CALLBACK(panel_applet_about_to_die),
-			  NULL);
-	g_signal_connect (G_OBJECT(panel),
+	g_signal_connect (G_OBJECT (panel),
 			  "back_change",
-			  G_CALLBACK(panel_back_change),
+			  G_CALLBACK (panel_back_change),
 			  NULL);
-	g_signal_connect (G_OBJECT(panel),
+	g_signal_connect (G_OBJECT (panel),
 			  "size_change",
-			  G_CALLBACK(panel_size_change),
+			  G_CALLBACK (panel_size_change),
 			  NULL);
 	g_signal_connect (G_OBJECT (panel),
 			  "orient_change",
@@ -1688,14 +1582,14 @@ panelw_size_alloc(BasePWidget *basep, GtkAllocation *alloc, gpointer data)
 	}
 }
 
-void
-panel_setup(GtkWidget *panelw)
+PanelData *
+panel_setup (GtkWidget *panelw)
 {
 	PanelData *pd;
 	BasePWidget *basep = NULL; 
 	PanelWidget *panel = NULL;
 
-	g_return_if_fail(panelw);
+	g_return_val_if_fail (panelw != NULL, NULL);
 
 	if (BASEP_IS_WIDGET (panelw)) {
 		basep = BASEP_WIDGET(panelw);
@@ -1799,6 +1693,8 @@ panel_setup(GtkWidget *panelw)
 		g_signal_connect_after(G_OBJECT(panelw), "realize",
 				       G_CALLBACK(panel_realize),
 				       NULL);
+
+	return pd;
 }
 
 /*send state change to all the panels*/
@@ -1837,26 +1733,27 @@ panel_data_by_id (const char *id)
 	return NULL;
 }
 
-void
-status_unparent (GtkWidget *widget)
+static void
+panel_ensure_panel_per_screen (int screen)
 {
-	GList *li;
-	PanelWidget *panel = NULL;
-	if (BASEP_IS_WIDGET (widget))
-		panel = PANEL_WIDGET(BASEP_WIDGET(widget)->panel);
-	else if (FOOBAR_IS_WIDGET (widget))
-		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
-	for(li=panel->applet_list;li;li=li->next) {
-		AppletData *ad = li->data;
-		AppletInfo *info = g_object_get_data (G_OBJECT (ad->applet),
-						      "applet_info");
-		if(info->type == APPLET_STATUS) {
-			status_applet_put_offscreen(info->data);
-		} else if(info->type == APPLET_DRAWER) {
-			Drawer *dr = info->data;
-			status_unparent(dr->drawer);
-		}
+	GtkWidget *panel;
+	GSList    *l;
+
+	for (l = panel_list; l; l = l->next) {
+		PanelData *pdata = l->data;
+
+		if (BASEP_IS_WIDGET (pdata->panel) &&
+		    BASEP_WIDGET (pdata->panel)->screen == screen)
+			return;
+
+		else if (FOOBAR_IS_WIDGET (pdata->panel) &&
+			 FOOBAR_WIDGET (pdata->panel)->screen == screen)
+			return;
 	}
+
+	panel = foobar_widget_new (NULL, screen, 0);
+	panel_save_to_gconf (panel_setup (panel));
+	gtk_widget_show (panel);
 }
 
 void
@@ -1866,17 +1763,15 @@ panel_load_global_config (void)
 
 	list = panel_gconf_all_global_entries ();
 
-	for (li = list; li != NULL ; li = li->next) {
-		panel_global_config_set_entry (li->data);
-		gconf_entry_free (li->data);
+	for (li = list; li != NULL; li = li->next) {
+		GConfEntry *entry = li->data;
+		li->data = NULL;
+		panel_global_config_set_entry (entry);
+		gconf_entry_free (entry);
 	}
 
 	g_slist_free (list);
 
-	/* FIXME STUFF THAT IS BORKED */
-	global_config.menu_check = TRUE;
-	global_config.menu_flags = get_default_menu_flags();
-
 	panel_apply_global_config ();
 }
 
@@ -1888,11 +1783,6 @@ panel_save_global_config (void)
 
 	change_set = gconf_change_set_new ();
 
-	/* FIXME STUFF THAT IS BORKED 
-	panel_gconf_global_config_set_int ("menu-flags", global_config.menu_flags);
-	panel_gconf_global_config_set_bool ("menu-check", global_config.menu_check);
-	*/
-
 	/*
 	 * keep in sync with panel-config-global.h and
 	 * panel-global-config.schemas
@@ -1937,15 +1827,6 @@ panel_save_global_config (void)
 	full_key = panel_gconf_global_key ("window_screenshot_key");
 	gconf_change_set_set_string (change_set, full_key, global_config.window_screenshot_key.str);
 
-	full_key = panel_gconf_global_key ("auto_raise_panel");
-	gconf_change_set_set_bool (change_set, full_key, global_config.autoraise);
-
-	full_key = panel_gconf_global_key ("panel_window_layer");
-	gconf_change_set_set_string (
-			change_set, full_key,
-			gconf_enum_to_string (panel_layer_type_enum_map, global_config.layer));
-
-
 	full_key = panel_gconf_global_key ("drawer_autoclose");
 	gconf_change_set_set_bool (change_set, full_key, global_config.drawer_auto_close);
 
@@ -1963,98 +1844,21 @@ panel_save_global_config (void)
 void
 panel_apply_global_config (void)
 {
-	static int layer_old = -1;
-	static int menu_flags_old = -1;
-	static int old_menu_check = -1;
-	GSList *li;
+	GSList *l;
 
 	if (global_config.tooltips_enabled)
 		gtk_tooltips_enable (panel_tooltips);
 	else
 		gtk_tooltips_disable (panel_tooltips);
-	/* not incredibly efficent way to do this, we just make
-	 * sure that all directories are reread */
-	if (old_menu_check != global_config.menu_check) {
-		fr_force_reread ();
-	}
-	if(old_menu_check != global_config.menu_check) {
-		GSList *li;
-		for(li = applets; li != NULL; li = g_slist_next(li)) {
-			AppletInfo *info = li->data;
-			if(info->menu != NULL) {
-				gtk_widget_unref(info->menu);
-				info->menu = NULL;
-				info->menu_age = 0;
-			}
-			if(info->type == APPLET_MENU) {
-				Menu *menu = info->data;
-				if(menu->menu != NULL) {
-					gtk_widget_unref(menu->menu);
-					menu->menu = NULL;
-					menu->age = 0;
-				}
-			}
-		}
-		for(li = panel_list; li != NULL; li = g_slist_next(li)) {
-			PanelData *pd = li->data;
-			if(pd->menu != NULL) {
-				gtk_widget_unref(pd->menu);
-				pd->menu = NULL;
-				pd->menu_age = 0;
-			}
-		}
-		foobar_widget_force_menu_remake();
-	}
-	old_menu_check = global_config.menu_check;
 
-	/* if we changed global menu flags, cmark all main menus that use
-	 * the global setting as dirty */
-	if(menu_flags_old != global_config.menu_flags) {
-		GSList *li;
-		for(li = applets; li != NULL; li = g_slist_next(li)) {
-			AppletInfo *info = li->data;
-			if(info->type == APPLET_MENU) {
-				Menu *menu = info->data;
-				if(menu->menu != NULL &&
-				   menu->global_main) {
-					gtk_widget_unref(menu->menu);
-					menu->menu = NULL;
-					menu->age = 0;
-				}
-			}
-		}
-	}
+	for (l = panel_list; l; l = l->next) {
+		PanelData *pd = l->data;
 
-	if (layer_old != global_config.layer) {
-		 for (li = panel_list; li != NULL; li = li->next) {
-			PanelData *pd = li->data;
-			if ( ! GTK_WIDGET_REALIZED (pd->panel))
-				continue;
-			if (BASEP_IS_WIDGET (pd->panel))
-				basep_widget_update_winhints (BASEP_WIDGET (pd->panel));
-			else if (FOOBAR_IS_WIDGET (pd->panel))
-				foobar_widget_update_winhints (FOOBAR_WIDGET (pd->panel));
-		}
-	}
-	layer_old = global_config.layer;
-	
-	for (li = panel_list; li != NULL; li = li->next) {
-		PanelData *pd = li->data;
-		if (BASEP_IS_WIDGET (pd->panel)) {
+		if (BASEP_IS_WIDGET (pd->panel))
 			basep_update_frame (BASEP_WIDGET (pd->panel));
-
-			if ((menu_flags_old != global_config.menu_flags) &&
-			    pd->menu) {
-				gtk_widget_unref (pd->menu);
-				pd->menu = NULL;
-				pd->menu_age = 0;
-			}
-				
-		}
 	}
-	menu_flags_old = global_config.menu_flags;
 
-	panel_global_keys_setup();
+	panel_global_keys_setup ();
 }
 
 
@@ -2148,20 +1952,21 @@ panel_set_bool (const char *profile,
 void
 panel_load_panels_from_gconf (void)
 {
-	GSList     *panels;
+	GSList     *panel_ids;
 	GSList     *l;
 	const char *profile;
 	const char *key;
+	int         i;
 
 	profile = panel_gconf_get_profile ();
 	
 	key = panel_gconf_general_key (profile, "panel_id_list");
 
-	panels = gconf_client_get_list (
+	panel_ids = gconf_client_get_list (
 			panel_gconf_get_client (), key, GCONF_VALUE_STRING, NULL);
 
-	for (l = panels; l; l = l->next) {
-		GtkWidget     *panel;
+	for (l = panel_ids; l; l = l->next) {
+		GtkWidget     *panel = NULL;
 		PanelType      type;
 		PanelBackType  back_type;
 		BasePState     state;
@@ -2173,6 +1978,7 @@ panel_load_panels_from_gconf (void)
 		gboolean       hidebuttons_enabled;
 		gboolean       hidebutton_pixmaps_enabled;
 		int            screen;
+		int            monitor;
 		int            size;
 		char          *panel_id;
 		char          *back_pixmap;
@@ -2180,9 +1986,21 @@ panel_load_panels_from_gconf (void)
 
 		panel_id = l->data;
 
-#ifdef PANEL_SESSION_DEBUG
-		printf ("Loading panel id %s\n", panel_id);
-#endif
+		screen  = panel_get_int (profile, panel_id, "screen", -1);
+		monitor = panel_get_int (profile, panel_id, "monitor", -1);
+		if (screen == -1 || monitor == -1) {
+			/* Backwards compat:
+			 *   In 2.0.0, we only had Xinerama support and the
+			 *   monitor number was saved as "screen_id".
+			 */
+			screen = 0;
+			monitor = panel_get_int (profile, panel_id, "screen_id", 0);
+		}
+
+		if (screen >= multiscreen_screens () ||
+		    monitor >= multiscreen_monitors (screen))
+			continue;
+
 		back_pixmap = panel_get_string (profile, panel_id,
 						"panel_background_pixmap", NULL);
 		if (string_empty (back_pixmap)) {
@@ -2197,7 +2015,7 @@ panel_load_panels_from_gconf (void)
 
 		tmp_str = panel_get_string (
 				profile, panel_id, "panel_background_type", "no_background");
-		gconf_string_to_enum (background_type_enum_map, tmp_str, (gint *) &back_type);
+		gconf_string_to_enum (background_type_enum_map, tmp_str, (int *) &back_type);
 		g_free (tmp_str);
 		
 		fit_pixmap_bg = panel_get_bool (profile, panel_id,
@@ -2219,9 +2037,8 @@ panel_load_panels_from_gconf (void)
 		hidebutton_pixmaps_enabled =
 			panel_get_bool (profile, panel_id, "hide_button_pixmaps_enabled", TRUE);
 
-		state  = panel_get_int (profile, panel_id, "panel_hide_state", 0);
-		mode   = panel_get_int (profile, panel_id, "panel_hide_mode", 0);
-		screen = panel_get_int (profile, panel_id, "screen_id", 0);
+		state   = panel_get_int (profile, panel_id, "panel_hide_state", 0);
+		mode    = panel_get_int (profile, panel_id, "panel_hide_mode", 0);
 
 		tmp_str = panel_get_string (profile, panel_id, "panel_type", "edge-panel");
 		gconf_string_to_enum (panel_type_type_enum_map, tmp_str, (gint *) &type);
@@ -2234,11 +2051,12 @@ panel_load_panels_from_gconf (void)
 			tmp_str = panel_get_string (profile, panel_id,
 						    "screen_edge", "panel-edge-bottom");
 			gconf_string_to_enum (
-				panel_edge_type_enum_map, tmp_str, (gint *) &edge);
+				panel_edge_type_enum_map, tmp_str, (int *) &edge);
 			g_free (tmp_str);
 			
 			panel = edge_widget_new (panel_id,
 						 screen,
+						 monitor,
 						 edge, 
 						 mode, state,
 						 size,
@@ -2258,17 +2076,18 @@ panel_load_panels_from_gconf (void)
 			tmp_str = panel_get_string (profile, panel_id,
 						    "screen_edge", "panel-edge-bottom");
 			gconf_string_to_enum (
-				panel_edge_type_enum_map, tmp_str, (gint *) &edge);
+				panel_edge_type_enum_map, tmp_str, (int *) &edge);
 			g_free (tmp_str);
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "panel_align", "panel-alignment-left");
 			gconf_string_to_enum (
-				panel_alignment_type_enum_map, tmp_str, (gint *) &align);
+				panel_alignment_type_enum_map, tmp_str, (int *) &align);
 			g_free (tmp_str);
 
 			panel = aligned_widget_new (panel_id,
 						    screen,
+						    monitor,
 						    align,
 						    edge,
 						    mode,
@@ -2291,19 +2110,20 @@ panel_load_panels_from_gconf (void)
 			tmp_str = panel_get_string (profile, panel_id,
 						    "screen_edge", "panel-edge-bottom");
 			gconf_string_to_enum (
-				panel_edge_type_enum_map, tmp_str, (gint *) &edge);
+				panel_edge_type_enum_map, tmp_str, (int *) &edge);
 			g_free (tmp_str);
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "panel_anchor", "panel-anchor-left");
 			gconf_string_to_enum (
-				panel_anchor_type_enum_map, tmp_str, (gint *) &anchor);
+				panel_anchor_type_enum_map, tmp_str, (int *) &anchor);
 			g_free (tmp_str);
 			
 			offset = panel_get_int (profile, panel_id, "panel_offset", 0);
 	
 			panel = sliding_widget_new (panel_id,
 						    screen,
+						    monitor,
 						    anchor,
 						    offset,
 						    edge,
@@ -2325,10 +2145,12 @@ panel_load_panels_from_gconf (void)
 			tmp_str = panel_get_string (profile, panel_id,
 						    "panel_orient", "panel-orient-up");
 			gconf_string_to_enum (
-				panel_orient_type_enum_map, tmp_str, (gint *) &orient);
+				panel_orient_type_enum_map, tmp_str, (int *) &orient);
 			g_free (tmp_str);
 
 			panel = drawer_widget_new (panel_id,
+						   screen,
+						   monitor,
 						   (PanelOrient) orient,
 						   BASEP_EXPLICIT_HIDE, 
 						   state,
@@ -2349,7 +2171,7 @@ panel_load_panels_from_gconf (void)
 			tmp_str = panel_get_string (profile, panel_id,
 						    "panel_orient", "panel-orientation-horizontal");
 			gconf_string_to_enum (
-				panel_orientation_type_enum_map, tmp_str, (gint *) &orient);
+				panel_orientation_type_enum_map, tmp_str, (int *) &orient);
 			g_free (tmp_str);
 
 			x = panel_get_int (profile, panel_id, "panel_x_position", 0);
@@ -2357,6 +2179,7 @@ panel_load_panels_from_gconf (void)
 			
 			panel = floating_widget_new (panel_id,
 						     screen,
+						     monitor,
 						     x,
 						     y,
 						     orient,
@@ -2373,10 +2196,9 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case FOOBAR_PANEL:
-			panel = foobar_widget_new (panel_id, screen);
+			panel = foobar_widget_new (panel_id, screen, monitor);
 			break;
 		default:
-			panel = NULL;
 			g_warning ("Unkown panel type: %d; ignoring.", type);
 			break;
 		}
@@ -2390,20 +2212,24 @@ panel_load_panels_from_gconf (void)
 	}
 
 	/* Failsafe! */
-	if (!panels) {
+	if (panel_ids == NULL) {
 		GtkWidget *panel;
 
-		panel_error_dialog ("no_panels_found",
-				    _("No panels were found in your "
-				      "configuration.  I will create "
-				      "a menu panel for you"));
+		panel_error_dialog (
+			gdk_screen_get_default (),
+			"no_panels_found",
+			_("No panels were found in your configuration.  "
+			  "I will create a menu panel for you"));
 
-		panel = foobar_widget_new (NULL, 0);
-		panel_setup (panel);
+		panel = foobar_widget_new (NULL, 0, 0);
+		panel_save_to_gconf (panel_setup (panel));
 		gtk_widget_show (panel);
 	}
 
-	panel_g_slist_deep_free (panels);
+	for (i = 0; i < multiscreen_screens (); i++)
+		panel_ensure_panel_per_screen (i);
+
+	panel_g_slist_deep_free (panel_ids);
 }
 
 void
@@ -2417,15 +2243,21 @@ panel_save_to_gconf (PanelData *pd)
 	const gchar *profile;
 	const char  *key;
 	char	    *color;
+	int          screen = 0;
+	int          monitor = 0;
 
 	g_return_if_fail (pd != NULL);
 	
 	if (BASEP_IS_WIDGET (pd->panel)) {
-		basep = BASEP_WIDGET (pd->panel);
-		panel = PANEL_WIDGET (basep->panel);
+		basep   = BASEP_WIDGET (pd->panel);
+		panel   = PANEL_WIDGET (basep->panel);
+		screen  = basep->screen;
+		monitor = basep->monitor;
 
 	} else if (FOOBAR_IS_WIDGET (pd->panel)) {
-		panel = PANEL_WIDGET (FOOBAR_WIDGET (pd->panel)->panel);
+		panel   = PANEL_WIDGET (FOOBAR_WIDGET (pd->panel)->panel);
+		screen  = FOOBAR_WIDGET (pd->panel)->screen;
+		monitor = FOOBAR_WIDGET (pd->panel)->monitor;
 	}
 
 	profile = panel_gconf_get_profile ();
@@ -2436,7 +2268,7 @@ panel_save_to_gconf (PanelData *pd)
 				client, key, GCONF_VALUE_STRING, NULL);
 
 	for (l = panel_id_list; l; l = l->next)
-		if (!strcmp (panel->unique_id, l->data))
+		if (strcmp (panel->unique_id, l->data) == 0)
 			break;
 	
 	if (!l) {
@@ -2453,6 +2285,9 @@ panel_save_to_gconf (PanelData *pd)
 	panel_set_string (profile, panel->unique_id, "panel_type", 
 			  gconf_enum_to_string (panel_type_type_enum_map, pd->type));
 
+	panel_set_int (profile, panel->unique_id, "screen", screen);
+	panel_set_int (profile, panel->unique_id, "monitor", monitor);
+
 	if (basep) {
 		panel_set_bool (profile, panel->unique_id,
 				"hide_buttons_enabled",
@@ -2467,9 +2302,6 @@ panel_save_to_gconf (PanelData *pd)
 
 		panel_set_int (profile, panel->unique_id,
 			       "panel_hide_state", basep->state);
-
-		panel_set_int (profile, panel->unique_id,
-			       "screen_id", basep->screen);
 	}
 
 	panel_set_string (profile, panel->unique_id, "panel_size", 
@@ -2544,10 +2376,6 @@ panel_save_to_gconf (PanelData *pd)
 				  gconf_enum_to_string (panel_orient_type_enum_map,
 							DRAWER_POS (basep->pos)->orient));
 		break;
-	case FOOBAR_PANEL:
-		panel_set_int (profile, panel->unique_id, 
-			       "screen_id", FOOBAR_WIDGET (pd->panel)->screen);
-		break;
 	default:
 		break;
 	}
@@ -2558,16 +2386,18 @@ panel_remove_from_gconf (PanelWidget *panel)
 {
 	const char *key;
 	GSList     *new_panels = NULL;
-	GSList     *panels;
+	GSList     *panel_ids;
 	GSList     *l;
 
 	key = panel_gconf_general_key (panel_gconf_get_profile (), "panel_id_list");
 
-	panels = gconf_client_get_list (
+	panel_ids = gconf_client_get_list (
 			panel_gconf_get_client (), key, GCONF_VALUE_STRING, NULL);
 
-	for (l = panels; l; l = l->next) {
-		if (!strcmp (panel->unique_id, (char *) l->data)) {
+	for (l = panel_ids; l; l = l->next) {
+		char *id = l->data;
+		l->data = NULL;
+		if (strcmp (panel->unique_id, id) == 0) {
 			char *dir;
 
 			dir = g_strdup_printf ("/apps/panel/profiles/%s/panels/%s",
@@ -2576,12 +2406,13 @@ panel_remove_from_gconf (PanelWidget *panel)
 
 			panel_gconf_clean_dir (panel_gconf_get_client (), dir);
 			g_free (dir);
-			g_free (l->data);
-		} else
-			new_panels = g_slist_prepend (new_panels, l->data);
+			g_free (id);
+		} else {
+			new_panels = g_slist_prepend (new_panels, id);
+		}
         }
 
-	g_slist_free (panels);
+	g_slist_free (panel_ids);
 
         gconf_client_set_list (panel_gconf_get_client (), key,
 			       GCONF_VALUE_STRING, new_panels, NULL);
@@ -2604,3 +2435,84 @@ panel_register_window_icon (void)
 		g_free (panel_icon);
 	}
 }
+
+GdkScreen *
+panel_screen_from_toplevel (GtkWidget *panel)
+{
+	GdkScreen  *retval = NULL;
+	GdkDisplay *display;
+
+	display = gdk_display_get_default ();
+
+	if (BASEP_IS_WIDGET (panel))
+	        retval = gdk_display_get_screen (
+	                        display, BASEP_WIDGET (panel)->screen);
+
+	else if (FOOBAR_IS_WIDGET (panel))
+	        retval = gdk_display_get_screen (
+	                        display, FOOBAR_WIDGET (panel)->screen);
+
+	return retval;
+}
+
+int
+panel_monitor_from_toplevel (GtkWidget *panel)
+{
+	int retval = -1;
+
+	if (BASEP_IS_WIDGET (panel))
+	        retval = BASEP_WIDGET (panel)->monitor;
+
+	else if (FOOBAR_IS_WIDGET (panel))
+	        retval = FOOBAR_WIDGET (panel)->monitor;
+
+	return retval;
+}
+
+int
+panel_screen_from_panel_widget (PanelWidget *panel)
+{
+	g_return_val_if_fail (PANEL_IS_WIDGET (panel), 0);
+	g_return_val_if_fail (panel->panel_parent != NULL, 0);
+	g_return_val_if_fail (BASEP_IS_WIDGET (panel->panel_parent) ||
+			      FOOBAR_IS_WIDGET (panel->panel_parent), 0);
+
+	if (BASEP_IS_WIDGET (panel->panel_parent))
+		return BASEP_WIDGET (panel->panel_parent)->screen;
+	else
+		return FOOBAR_WIDGET (panel->panel_parent)->screen;
+}
+
+int
+panel_monitor_from_panel_widget (PanelWidget *panel)
+{
+	g_return_val_if_fail (PANEL_IS_WIDGET (panel), 0);
+	g_return_val_if_fail (panel->panel_parent != NULL, 0);
+	g_return_val_if_fail (BASEP_IS_WIDGET (panel->panel_parent) ||
+			      FOOBAR_IS_WIDGET (panel->panel_parent), 0);
+
+	if (BASEP_IS_WIDGET (panel->panel_parent))
+		return BASEP_WIDGET (panel->panel_parent)->monitor;
+	else
+		return FOOBAR_WIDGET (panel->panel_parent)->monitor;
+}
+
+gboolean
+panel_is_applet_right_stick (GtkWidget *applet)
+{
+        PanelWidget *panel;
+
+        g_return_val_if_fail (GTK_IS_WIDGET (applet), FALSE);
+        g_return_val_if_fail (PANEL_IS_WIDGET (applet->parent), FALSE);
+
+        panel = PANEL_WIDGET (applet->parent);
+
+        /* These types of panels are *always* packed */
+        if (ALIGNED_IS_WIDGET (panel->panel_parent) ||
+            BORDER_IS_WIDGET (panel->panel_parent)  ||
+            SLIDING_IS_WIDGET (panel->panel_parent) ||
+            FLOATING_IS_WIDGET (panel->panel_parent))
+                return FALSE;
+
+        return panel_widget_is_applet_stuck (panel, applet);
+}
